local needtoexit
local gamemapid
local mapstuckid
local maprules = { /*
	mapid = {"level load command(s)"}
	OR
	mapid = {"level load command(s)", "exit level command(s)"}
*/ }

local function MapIDtoINT(id)
    if (id:len() == 2) then -- Make sure it's a 2 character ID
        --check if directly convertible to number
        local idNum = tonumber(id)
        if idNum == nil then
            --if not, convert to number by exmap slot logic
            --check if second character is directly convertible to number
            idNum = tonumber(id:sub(2))
            if idNum == nil then
                --if not, use offset ASCII value
                idNum = id:byte(2) - 55
            end
            --add offset ASCII value of first character and offset agaib
            idNum = 36 * (id:byte(1) - 65) + $ + 100
        end
        return idNum
    end
end -- Close function

COM_AddCommand("hm_maprule_add", function(player, mapidinput, loadinput, exitinput)
	if not (mapidinput) then -- Handle no inputs
		CONS_Printf(player, "\nhm_maprule_add \130<Map ID>  <Level Load Command> \135<Level Exit Command>\n           \133*Required *Required             \134Optional\n")
		return
	end
	local cleanmapid = MapIDtoINT(mapidinput)
	if (mapheaderinfo[cleanmapid] == nil) then
		CONS_Printf(player, "\nPlease enter a valid Map ID. Your map may not be loaded.\n")
	else
		local contextwords = {"Added", "to"}
		if (maprules[cleanmapid]) then contextwords = {"Changed", "for"} end
		if (exitinput) then
			maprules[cleanmapid] = {loadinput, exitinput} -- Overwrite/add rule
			CONS_Printf(player, "\n" .. contextwords[1] .. " rules \130\"" .. maprules[cleanmapid][1] .. "\" \128and \130\"" .. maprules[cleanmapid][2] .. "\" \128" .. contextwords[2] .. " \135" .. mapheaderinfo[cleanmapid].lvlttl .. "\n")
		else
			maprules[cleanmapid] = {loadinput} -- Overwrite/add rule
			CONS_Printf(player, "\n" .. contextwords[1] .. " rule \130\"" .. maprules[cleanmapid][1] .. "\" \128" .. contextwords[2] .. " \135" .. mapheaderinfo[cleanmapid].lvlttl .. "\n")
		end
	end
end, 1) -- Close function

COM_AddCommand("hm_maprule_remove", function(player, mapidinput)
	if not (mapidinput) then -- Handle no inputs
		CONS_Printf(player, "\nhm_maprule_remove \130<Map ID>\n              \133*Required\n")
		return
	end
	local cleanmapid = MapIDtoINT(mapidinput)
	if not (maprules[cleanmapid]) then
		CONS_Printf(player, "\nThere are no rules set for this map.\n")
	else
		if (maprules[cleanmapid][2]) then
			CONS_Printf(player, "\nRemoved rules \130\"" .. maprules[cleanmapid][1] .. "\" \128and \130\"" .. maprules[cleanmapid][2] .. "\" \128from \135" .. mapheaderinfo[cleanmapid].lvlttl .. "\n")
			table.remove(maprules, cleanmapid) -- Remove rules from table
		else
			CONS_Printf(player, "\nRemoved rule \130\"" .. maprules[cleanmapid][1] .. "\" \128from map \135" .. mapheaderinfo[cleanmapid].lvlttl .. "\n")
			table.remove(maprules, cleanmapid) -- Remove rule from table
		end
	end
end, 1)

COM_AddCommand("hm_maprule_list", function(player)
	CONS_Printf(player, "\n") -- I like padding
	for k,v in pairs(maprules)
		if (maprules[k][2]) then
			CONS_Printf(player, "\135" .. mapheaderinfo[k].lvlttl .. " (Map " .. k .. ")\n\128Load: \130\"" .. maprules[k][1] .. "\"\n\128Exit: \130\"" .. maprules[k][2] .. "\"\n\n")
		else
			CONS_Printf(player, "\135" .. mapheaderinfo[k].lvlttl .. " (Map " .. k .. ")\n\128Load: \130\"" .. maprules[k][1] .. "\"\n\n")
		end
	end
end, 0)

local function ExecuteMapRule()
	gamemapid = MapIDtoINT(string.gsub(G_BuildMapName(), "MAP", ""))
	if (gamemapid == mapstuckid) then return end -- Bail if the same map comes up twice
	if (maprules[gamemapid]) then
		COM_BufInsertText(server, maprules[gamemapid][1])
		if (maprules[gamemapid][2]) then -- Check if exit command exists
			needtoexit = true -- We need to run that command
		end
	end
	mapstuckid = gamemapid
end -- Close function

local function ExecuteExitRule()
	if (needtoexit == true) then
		COM_BufInsertText(server, maprules[gamemapid][2])
		needtoexit = false -- We ran the exit command
	end
end -- Close function

local function UnlockStuckPrevention()
	if (leveltime == 350) then
		mapstuckid = nil
	end
end -- Close function

addHook("MapLoad",	ExecuteMapRule)
addHook("ThinkFrame",	UnlockStuckPrevention)
addHook("IntermissionThinker",	ExecuteExitRule)
addHook("VoteThinker",	ExecuteExitRule)

addHook("NetVars", function(n)
	maprules = n($)
	gamemapid = n($)
	mapstuckid = n($)
	needtoexit = n($)
end)