local hostmod_restat = CV_RegisterVar({
	name = "hm_restat",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
})
local hostmod_restatnotify = CV_RegisterVar({
	name = "hm_restat_notify",
	defaultvalue = "On",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
})

-- no, but actually, why isn't this just a base game constant?
local STARTTIME = 6 * TICRATE + (3 * TICRATE / 4)

-- common reused strings
local function statstring(s, w)
	return ""..s.." spd, "..w.." wt"
end
local function statusstring(p)
	if p.HMRs_p then
		return statstring(p.HMRs_p, p.HMRw_p)
	else
		return "using your skin's default stats"
	end
end
local function usage(p, str)
	CONS_Printf(p, str)
	local reminder = "restat random" -- if people already have stats set...
	if p.HMRs_p then reminder = "restat off" end -- it's likely they'll want to remove them
	CONS_Printf(p, "\140Try something like \135restat 2 9\140, \135restat tails\140, or \135"..reminder..".")
end

local function initstats(p)
	if p.HMRs == nil then
		p.HMRs = 0
		p.HMRw = 0
		p.HMRs_p = 0
		p.HMRw_p = 0
		return
	end
end

-- apply and announce stat change
local function updatestats(p)
	initstats(p)
	if p.HMRs_p != p.HMRs or p.HMRw_p != p.HMRw then
		p.HMRs = p.HMRs_p
		p.HMRw = p.HMRw_p
		if hostmod_restatnotify.value then
			if p.HMRs then
				chatprint("\140*"..p.name.." is now \130"..statstring(p.HMRs, p.HMRw).."\140.", true)
			else
				chatprint("\140*"..p.name.." returned to default stats.", true)
			end
		end
	end
end

-- whee, text command parsing! this is always the worst.
local function changestats(p, s, w, ...)
	-- maybe i should dynamically register and unregister the command in this situation?
	if not hostmod_restat.value then
		CONS_Printf(p, "\138This function has been disabled by the server host.")
		return
	end
	initstats(p)
	-- handle a potential name search first
	if s then
		local fug = s
		if w then fug = $.." "..w end
		if ... then fug = $..table.concat({...}, " ") end
		fug = $:lower()
		for v in skins.iterate
			if fug == v.name:lower() or fug == v.realname:lower() then
				s = v.kartspeed
				w = v.kartweight
			end
		end
	end
	-- catch the user doing something stupid
	if not (s and w) then
		if s == "random" then
			s = HM_RandomRange(1, 9)
			w = HM_RandomRange(1, 9)
			-- Better to do this with local random, but gotta sync it properly
			COM_BufAddText(p, "restat "..s.." "..w)
			return
		elseif s == "default" or s == "reset" or s == "off" then
			p.HMRs_p = 0
			p.HMRw_p = 0
			CONS_Printf(p, "\140Restat \130disabled\140. You will use the stats of your skin again.")
			if server.HMRint then
				updatestats(p)
			end
			return
		elseif s
			usage(p, "\140Couldn't find a matching skin.")
			return
		else
			usage(p, "\140You are currently \130"..statusstring(p).."\140.")
			return
		end
	end
	-- whee, sentinel values!
	s, w = tonumber($), tonumber($)
	if not (s or w) then
		usage(p, "\140Sorry, couldn't parse that.")
		return
	end
	if min(s, w) < 1 or max(s, w) > 9 then
		CONS_Printf(p, "\140Stats out of bounds. Legal values are 1-9.")
		return
	end
	-- update pending stats. we'll apply these for real when it's appropriate.
	p.HMRs_p = s
	p.HMRw_p = w
	CONS_Printf(p, "\140OK! You will be \130"..statstring(s, w).."\140 for the next race.")
	CONS_Printf(p, "\140Use \135restat off\140 to return to your skin's default stats.")
	if server.HMRint then
		updatestats(p)
	end
end
COM_AddCommand("restat", changestats)

addHook("MapChange", function()
	server.HMRint = 0
end)

addHook("ThinkFrame", function()
	if not hostmod_restat.value then return end
	HM_Random()
	for p in players.iterate
		if leveltime == 0 then
			updatestats(p)
		end
		-- announce
	    if not p.HMRwarned then
            p.HMRwarned = true
            --chatprintf(p, "\140*Server running \130Restat\140. Use \135restat\140 in console to change your stats.")
        end
		-- fuck off if we need to fuck off
		if p.spectator then continue end
		-- lock our stats
		if p.HMRs then
			p.kartspeed = p.HMRs
			p.kartweight = p.HMRw
		else
			p.kartspeed = skins[p.mo.skin].kartspeed
			p.kartweight = skins[p.mo.skin].kartweight
		end
	end
end)