local hm_veto = CV_RegisterVar({
	name = "hm_veto",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
})
local hm_veto_threshold = CV_RegisterVar({
	name = "hm_veto_threshold",
	defaultvalue = "50",
	flags = CV_NETVAR,
	possiblevalue = CV_Natural
})
local hm_veto_hellopen = CV_RegisterVar({
	name = "hm_veto_hellopen",
	defaultvalue = "100",
	flags = CV_NETVAR,
	possiblevalue = CV_Unsigned
})
local hm_veto_hellclosed = CV_RegisterVar({
	name = "hm_veto_hellclosed",
	defaultvalue = "50",
	flags = CV_NETVAR,
	possiblevalue = CV_Unsigned
})


local STARTTIME = 6 * TICRATE + (3 * TICRATE / 4)
local VETOTIME = 20*TICRATE

local VS_VOTING, VS_SUCCESS, VS_FAILURE, VS_DONE = 1,2,3,4
local HL_NONE, HL_OPEN, HL_CLOSED = 1,2,3

local function votetoveto(p)
	if not p.mo and p.mo.valid then return false end
	if not server.HMvetostate == VS_VOTING then return false end
	if not p.HMvetovote then
		S_StartSound(nil, sfx_s257)
		chatprintf(p, "\131*You voted to veto this track.", true)
		p.HMvetovote = true
		return true
	end
end

local function vschange(state)
	server.HMvetostate = state
	server.HMvetotransition = 0
end

local function hell()
	if server.HMhellopen and hm_veto_hellopen.value and server.HMhellopen == hm_veto_hellopen.value then
		return HL_OPEN
	elseif server.HMhellclosed and hm_veto_hellclosed.value and server.HMhellclosed == hm_veto_hellclosed.value then
		return HL_CLOSED
	end
	return HL_NONE
end

addHook("ThinkFrame", function()
	if not hm_veto.value then return end
	if leveltime == 0 then return end
	if leveltime == 1 then
		vschange(VS_VOTING)
		for p in players.iterate
			p.HMvetovote = false
		end
		server.HMhellopen = P_RandomRange(1, hm_veto_hellopen.value)
		server.HMhellclosed = P_RandomRange(1, hm_veto_hellclosed.value)
		--print(server.HMhellopen, server.HMhellclosed)
		if hell() == HL_OPEN and not server.HMvetolock then
			S_StartSound(nil, sfx_s3k9f)
		end
	end
	if server.HMvetolock then
		vschange(VS_DONE)
		return
	end
	if server.HMvetostate == VS_VOTING then
		local ca = 0
		local cv = 0
		for p in players.iterate
			if not (p.mo and p.mo.valid) then continue end
			ca = $ + 1
			p.HMvetovote = $ or false
			p.HMvetotrigger = $ or 0
			local bt = p.cmd.buttons
			if (bt & BT_ACCELERATE) and (bt & BT_BRAKE) and (bt & BT_ATTACK) and (bt & BT_DRIFT) then
				p.HMvetotrigger = $ + 1
				if p.HMvetotrigger == TICRATE/2 then votetoveto(p) end
			else
				p.HMvetotrigger = 0
			end
			if p.HMvetovote then
				cv = $ + 1
			end
		end
		if ca > 0 and cv * 100 / ca > hm_veto_threshold.value then
			vschange(VS_SUCCESS)
			if hell() == HL_OPEN then
				S_StartSound(nil, sfx_noooo2)
			elseif hell() == HL_CLOSED then
				S_StartSound(nil, sfx_noooo1)
			else
				S_StartSound(nil, sfx_s1a1)
			end
		elseif leveltime > VETOTIME then
			local loud = false
			for p in players.iterate
				if not (p.mo and p.mo.valid) then continue end
				if p.HMvetovote then 
					loud = true
					S_StartSound(nil, sfx_s231, p)
				end
			end
			if loud then
				vschange(VS_FAILURE)
			else
				vschange(VS_DONE)
			end
		end
	else
		if server.HMvetostate == VS_SUCCESS and server.HMvetotransition == TICRATE*3 then
			server.HMvetolock = true
			if hell() == HL_NONE then
				COM_BufAddText(server, "randommap")
			else
				local maps = {}
				for i=1,#mapheaderinfo do
					if mapheaderinfo[i] and mapheaderinfo[i].lvlttl
					and mapheaderinfo[i].typeoflevel == mapheaderinfo[gamemap].typeoflevel and mapheaderinfo[i].menuflags & LF2_HIDEINMENU
					and i != 1035 then
						table.insert(maps, i)
					end
				end
				COM_BufAddText(server, "map "..G_BuildMapName(maps[P_RandomRange(1, #maps)]))
			end
		elseif server.HMvetostate == VS_FAILURE and server.HMvetotransition == TICRATE*3 then
			vschange(VS_DONE)
		end
	end
	server.HMvetotransition = $ or 0
	server.HMvetotransition = $ + 1
end)

addHook("PlayerMsg", function(source, msgtype, target, msg)
	if (not hm_veto.value) or msgtype != 0 then
		return
	end
	msg = string.lower(msg)
	if msg == "rtv" then
		return votetoveto(source)
	end
end)

local function youareallfreenow()
	server.HMvetolock = false
	server.HMvetostate = VS_DONE
end
addHook("IntermissionThinker", youareallfreenow)
addHook("IntermissionThinker", youareallfreenow)

local function drawStringCenter(v, xx, yy, ss, ff, tt)
	local ww = v.stringWidth(ss, ff, tt)
	v.drawString(xx - (ww / 2), yy, ss, ff, tt)
end

hud.add(function(v, p, c)
	if leveltime == 0 then return end
	if not hm_veto.value then return end
	if not server.HMvetostate then return end
	if replayplayback then return end
	local fl = V_ALLOWLOWERCASE|V_SNAPTOBOTTOM
	if server.HMvetotransition and server.HMvetotransition < TICRATE and (leveltime/2%2) then
		fl = $|V_40TRANS
	end
	if server.HMvetostate != VS_DONE then
		-- v.drawString(2, 178, "VETO?", fl, "thin")
		local st = "chat \130rtv\128 or \130hold all 4 buttons\128 to \133veto\128 this course"
		if server.HMvetostate == VS_SUCCESS then
			if hell() != HL_NONE then
				if leveltime%2 then
					st = "\131veto succeeded\128... the gates of \133MAP HELL\128 open!"
				else
					st = "\131veto succeeded\128... the gates of \130MAP HELL\128 open!"
				end
			else
				st = "\131veto succeeded\128, bailing out..."
			end
		elseif server.HMvetostate == VS_FAILURE then
			if hell() == HL_OPEN then
				st = "\133veto failed\128. somewhere, a door closes...?"
			else
				st = "\133veto failed\128, not enough votes"
			end
		elseif p.HMvetotrigger and p.HMvetotrigger > 0 and (not p.HMvetovote) and (splitscreen == 0) then
			st = "\130KEEP HOLDING..."
		elseif hell() == HL_OPEN then
			if (leveltime/17)%2 then
				st = "A \133forboding wind\128 is blowing...?!"
			else
				st = "A \130forboding wind\128 is blowing...?!"
			end
		end
		drawStringCenter(v, 160, 192, st, fl, "small")
		local ca = 0
		local ma = 0
		local cv = 0
		for p in players.iterate
			if not (p.mo and p.mo.valid) then continue end
			ma = $ + 1
		end
		for p in players.iterate
			if not (p.mo and p.mo.valid) then continue end
			ca = $ + 1
			if p.HMvetovote then
				cv = $ + 1
			end
			local pp = skins[p.mo.skin].facerank
			local cmap = v.getColormap(p.mo.skin, p.mo.color)
			local dieflag = 0
			if p.mo.colorized then cmap = v.getColormap(TC_RAINBOW, p.mo.color) end
			if not p.HMvetovote then
				cmap = v.getColormap(TC_RAINBOW, SKINCOLOR_GREY)
				dieflag = V_50TRANS
			end
			v.drawScaled((160 + 10*ca - (10*ma/2))<<FRACBITS, (182)<<FRACBITS, FRACUNIT/2, v.cachePatch(pp), dieflag|fl, cmap)
		end
	end
end)