local hostmod_votetimer = CV_RegisterVar({
	name = "hm_vote_timer",
	defaultvalue = "30",
	flags = CV_NETVAR,
	possiblevalue = CV_Natural
})
local hostmod_autopass = CV_RegisterVar({
	name = "hm_vote_autopass",
	defaultvalue = "50",
	flags = CV_NETVAR,
	possiblevalue = CV_Natural
})
local hm_vote_allowidc = CV_RegisterVar({
	name = "hm_vote_allowidc",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
})

-- Constants
local VOTE_YES = 1
local VOTE_NO = -1
local VOTE_IDC = 0

local VTYPE_COMMAND = 0
local VTYPE_KICK = 1

local inrace = false

-- We need a whitelist, or a whole bunch of difficult-to-anticipate malicious shit becomes possible.
local function votable(p, ...)
	if not ... then
		CONS_Printf(p, "Supply a command to be whitelisted for vote.")
		return
	end
	if not server.HMvotable then server.HMvotable = {} end
	local target = table.concat({...}, " ")
	if target:match(" ") then
		CONS_Printf(p, "Supply just the root command, please.")
		return
	end
	table.insert(server.HMvotable, target)
	CONS_Printf(p, "Added \130"..target.."\128 to the votable list.")
end
COM_AddCommand("hm_votable", votable, 1)

local function playercount()
	local a = 0
	for p in players.iterate
		a = $ + 1
	end
	return a
end

local function totalvote(num)
	local a = 0
	for p in players.iterate
		if p.HMvote == num then a = $ + 1 end
	end
	return a
end

local function votestats()
	if hm_vote_allowidc.value then
		return "("..totalvote(VOTE_YES).." yes, "..totalvote(VOTE_NO).." no, "..totalvote(VOTE_IDC).." didn't care)"
	else
		return "("..totalvote(VOTE_YES).." yes, "..totalvote(VOTE_NO).." no)"
	end
end

-- These just pass to the thinkframe to handle vote resolution, they're purely cosmetic otherwise.
local function passvote()
	server.HMvtimer = 0
	server.HMvresolve = 3 * TICRATE / 2
	if server.HMvtype == VTYPE_KICK and consoleplayer == server.HMvkicktarget then return end
	S_StartSound(nil, sfx_s1a1)
	chatprint("\131*Vote passed! "..votestats())
end

local function failvote()
	S_StartSound(nil, sfx_s231)
	chatprint("\131*Vote failed. "..votestats())
	server.HMvtimer = 0
end

local function resolvevote()
	if not hm_vote_allowidc.value then
		hm_dprint("the goddamn no-vote conversion")
		for p in players.iterate
			hm_dprint("inside,", p.name, p.HMvote)
			if p.HMvote == VOTE_IDC or p.HMvote == nil then
				p.HMvote = VOTE_NO
				hm_dprint("did the thing")
			end
		end
	end
	if totalvote(VOTE_YES) > totalvote(VOTE_NO) then
		passvote()
	else
		failvote()
	end
	server.HMvtimer = 0
end

-- Abstraction~
local function votefx(num)
	if not inrace then return end
	if server.HMvtype == VTYPE_KICK and consoleplayer == server.HMvkicktarget then return end
	S_StartSound(nil, sfx_s257)
end

-- Resolve the vote early in cases of positive landslides or everyone voting.
local function earlycheck()
	if playercount() < 1 then return end
	if ((totalvote(VOTE_YES) * 100) / (playercount())) > hostmod_autopass.value then
		resolvevote()
		return
	end
	if totalvote(VOTE_YES) + totalvote(VOTE_NO) == playercount() then
		resolvevote()
		return
	end
end

local function castvote(p, vote)
	if not server.HMvtimer then
		chatprintf(p, "\131*There's no vote running...")
		return
	end
	votefx(vote)
	if not p.HMvote or p.HMvote == VOTE_IDC then
		if not (server.HMvtype == VTYPE_KICK and consoleplayer == server.HMvkicktarget) then
			chatprintf(p, "\131*Your vote was cast.", true)
		end
	end
	p.HMvote = vote
	CONS_Printf(server, p.name.." voted "..vote)
	earlycheck()
end

-- Bindable console commands.
local function voteyes(p, ...)
	castvote(p, VOTE_YES)
end
COM_AddCommand("voteyes", voteyes)

local function voteno(p, ...)
	castvote(p, VOTE_NO)
end
COM_AddCommand("voteno", voteno)

local function voteidc(p, ...)
	castvote(p, VOTE_IDC)
end
COM_AddCommand("voteidc", voteidc)

local function printwl(p)
	if server.HMvotable == nil then
		CONS_Printf(p, "This server hasn't enabled any votable commands.")
		return
	end
	CONS_Printf(p, "Whitelisted commands are: \130"..table.concat(server.HMvotable, "\128, \130"))
end

-- Actually handle calling a vote.
local function initvote(timer, command, initiator)
	if initiator.HMvcd then
		local inseconds = initiator.HMvcd / 35 + 1
		CONS_Printf(initiator, "Wait a bit before calling a vote. ("..inseconds.." seconds left.)")
		return
	end
	if server.HMvtimer or server.HMvresolve then
		CONS_Printf(initiator, "A vote is already in progress. Wait until it's finished!")
		return
	end
	if command:match(";") then -- Prevents whitelist bypasses.
		CONS_Printf(initiator, "Nice Try!")
		return
	end
	
	server.HMvkicktarget = nil
	server.HMvname = initiator
	server.HMvcommand = command:gsub("'", '"') -- Let's at least theoretically let people pass in quotes...
	server.HMvtype = VTYPE_COMMAND
	
	-- Find what "base" command is being invoked and check it against the whitelist.
	local spaceindex = string.find(server.HMvcommand, " ", 0, yes)
	local commandfrag = server.HMvcommand
	if spaceindex then
		commandfrag = server.HMvcommand:sub(0, spaceindex - 1)
	end
	local whitelisted = false
	if not server.HMvotable then server.HMvotable = {} end
	for _,v in pairs(server.HMvotable)
		if v == commandfrag then
			whitelisted = true
		end
	end
	if not whitelisted then
		CONS_Printf(initiator, "That command isn't whitelisted for voting.")
		printwl(initiator)
		return
	end
	
	-- If we're doing kick stuff, handle some specific syntax sugar...
	if commandfrag == "kick" then
		local target = command:sub(6):lower()
		local acc = 0
		-- We want to mark a player so they can't dodge by name.
		-- We also want them to be targetable by number to avoid griefing with barcode names.
		-- (e.g. "Player 2 renamed to Ii1iLi1iliIl1iililLi1l")
		if tonumber(target) != nil then
			server.HMvkicktarget = players[tonumber(target)] -- thanks, terminal!
		else
			for p in players.iterate
				if p.name:lower():match(target) then
					acc = $ + 1
					server.HMvkicktarget = p
				end
			end
		end
		if server.HMvkicktarget == nil then
			CONS_Printf(initiator, "No player matching that name / number exists, check your spelling?")
			return
		end
		if acc > 1 then -- Too dangerous to allow ambiguity.
			CONS_Printf(initiator, "That kick attempt matches more than one player. Be more specific!")
			return
		end
		if IsPlayerAdmin(server.HMvkicktarget) or server.HMvkicktarget == server then
			if hm_debug.value then
				CONS_Printf(initiator, "...I'll let that slide because you're in testing mode.")
			else
				server.HMvkicktarget = initiator
			end
		end
		server.HMvcommand = server.HMvkicktarget.name
		server.HMvtype = VTYPE_KICK
	end
	
	server.HMvtimer = timer * TICRATE
	initiator.HMvcd = (hostmod_votetimer.value + 10) * TICRATE
	
	for p in players.iterate
		p.HMvote = VOTE_IDC
	end
	local guide = (server.HMvkicktarget != nil) and "kick " or ""
	if not (server.HMvtype == VTYPE_KICK and consoleplayer == server.HMvkicktarget) then
		S_StartSound(nil, sfx_s258)
		chatprint("\131*"..initiator.name.." called a vote to \130"..guide..server.HMvcommand.."\131. Vote in chat with \130y \131/ \130n\131.")
	end
	if not hm_debug.value then -- In debug mode, let's allow the vote UI to show up even if there's only one person.
		castvote(initiator, VOTE_YES)
	end
	if server.HMvtype == VTYPE_KICK then
		castvote(server.HMvkicktarget, VOTE_NO)
	end
end

local function vote(p, ...)
	if not ... then
		CONS_Printf(p, "What are you trying to vote for?")
		printwl(p)
		return
	end
	local target = table.concat({...}, " ")
	initvote(hostmod_votetimer.value, target, p)
end
COM_AddCommand("vote", vote)

-- Syntax sugar.
local function votekick(p, ...)
	if ... then
		COM_BufAddText(p, "vote kick "..table.concat({...}, " "))
	else
		-- Let's not even bother passing garbage to the main handler.
		CONS_Printf(p, "No player matching that name / number exists, check your spelling?")
	end
end
COM_AddCommand("votekick", votekick)

local function rtv(p, ...)
	COM_BufAddText(p, "vote exitlevel")
end
COM_AddCommand("rtv", rtv)

addHook("PlayerMsg", function(source, msgtype, target, msg)
	if msgtype != 0 then return end
	if (msg:sub(1, 3):lower() != "rtv") or (CV_FindVar("hm_veto").value) then return end
	if server.HMvtimer then
		COM_BufAddText(source, "voteyes")
		return true
	else
		COM_BufAddText(source, "vote exitlevel")
		return true
	end
end)

-- Vote timer and resolution handler.
local function tickVote()
	for p in players.iterate
		if p.HMvcd == nil then
			p.HMvcd = 30*TICRATE
			continue
		end
		p.HMvcd = $ or 0
		p.HMvcd = max(0, $-1)
	end
	if server.HMvresolve then
		server.HMvresolve = $ - 1
		if not server.HMvresolve then
			if server.HMvtype == VTYPE_COMMAND then
				COM_BufAddText(server, server.HMvcommand)
			elseif server.HMvtype == VTYPE_KICK then
				if server.HMvkicktarget and server.HMvkicktarget.valid then
					COM_BufAddText(server, 'kick "'..server.HMvkicktarget.name..'"')
				else
					chatprint("\131* ...but the target wasn't here to be kicked!")
				end
			end
		end
	end
	if not server.HMvtimer then return end
	server.HMvtimer = $ - 1
	for p in players.iterate
		if not p.HMvote then p.HMvote = VOTE_IDC end
	end
	if not server.HMvtimer then
		resolvevote()
	end
end

addHook("IntermissionThinker", tickVote)
addHook("ThinkFrame", tickVote)
addHook("VoteThinker", tickVote)

addHook("ThinkFrame", function()
	inrace = true
end)
local function notInRace()
	inrace = false
end
addHook("IntermissionThinker", notInRace)
addHook("VoteThinker", notInRace)

-- HUD
local function drawStringCenter(v, xx, yy, ss, ff, tt)
	local ww = v.stringWidth(ss, ff, tt)
	v.drawString(xx - (ww / 2), yy, ss, ff, tt)
end

hud.add(function(v, p, c)
	if not server.HMvtimer then return end
	
	if server.HMvtype == VTYPE_KICK and consoleplayer == server.HMvkicktarget then return end
	
	local flags = V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_ALLOWLOWERCASE
	local roundup = server.HMvtimer + 35
	local tx = 250 -- Top X
	local ty = 130 -- Left Y
	local bx = 310 -- Bottom X
	local by = 190 -- Right Y
	local brd = 2 -- Border
	local pad = 4 -- Padding
	v.drawFill(tx, ty, bx-tx, by-ty, 31|flags)
	v.drawFill(tx+brd, ty+brd, bx-tx-brd-brd, by-ty-brd-brd, 207|flags)
	local guide = (server.HMvtype == VTYPE_KICK) and "KICK?" or "VOTE"
	drawStringCenter(v, (bx+tx)/2, ty+pad, guide, flags)
	v.drawFill(tx, ty+pad+12-brd, bx-tx, 5+brd, 31|flags)
	drawStringCenter(v, (bx+tx)/2, ty+pad+12, server.HMvcommand, flags|V_YELLOWMAP, "small")
	v.drawString(tx+pad, ty+pad+20, 'YEAH', flags|V_GREENMAP, "small")
	v.drawString(bx-pad, ty+pad+20, 'NAH', flags|V_REDMAP, "small-right")
	v.drawString(tx+pad, ty+pad+25, totalvote(VOTE_YES), flags|V_GREENMAP)
	v.drawString(bx-pad, ty+pad+25, totalvote(VOTE_NO), flags|V_REDMAP, "right")
	drawStringCenter(v, (bx+tx)/2, ty+pad+35, 'chat \131y\128 or \133n', flags|V_50TRANS, "small")
	v.drawString(bx-pad, by-5-pad, G_TicsToMinutes(roundup)..":"..string.format("%02d", G_TicsToSeconds(roundup)), flags, "small-right")
end)

-- Chat vote handler
addHook("PlayerMsg", function(source, msgtype, target, msg)
	if not server.HMvtimer or msgtype != 0 then
		return
	end
	msg = string.lower(msg)
	if msg == "y" then
		castvote(source, VOTE_YES)
		return true
	end
	if msg == "n" then
		castvote(source, VOTE_NO)
		return true
	end
	if msg == "idc" or msg == "dc" or msg == "j" then
		castvote(source, VOTE_IDC)
		return true
	end
end)